#!/usr/bin/python
# xDevs.com DCV Linearity test application
# Python 2.7 
# Depends on python-vxi11 library to access GPIB instruments
import sys
import vxi11
import time
import numpy as np

log_name = "dcl_10vdc_abc_abe_hulk1_11rng_resl8_nplc10_soak10.csv"

fluke = vxi11.Instrument("192.168.1.6", "gpib0,1") # Fluke 5720A, GPIB Address = 1
fluke.timeout = 15

dmm = vxi11.Instrument("192.168.1.6", "gpib0,2") # Keysight 3458A, GPIB Address = 2
dmm.timeout = 15
dmm.write("RESET")
dmm.clear()

dmm2 = vxi11.Instrument("192.168.1.6", "gpib0,3") # Keysight 3458A, GPIB Address = 3
dmm2.timeout = 15
dmm2.write("RESET")
dmm2.clear()

dmm3 = vxi11.Instrument("192.168.1.6", "gpib0,5") # Keysight 3458A, GPIB Address = 5
dmm3.timeout = 15
dmm3.write("RESET")
dmm3.clear()

dmm4 = vxi11.Instrument("192.168.1.6", "gpib0,11") # Keysight 3458A, GPIB Address = 11
dmm4.timeout = 15
dmm4.write("RESET")
dmm4.clear()

fluke.write("STBY")
fluke.write("RANGELCK OFF")
fluke.write("OUT 10 V, 0 Hz")
fluke.write("RANGELCK ON")            # Lock range on source to avoid auto-range errors
fluke.write("STBY")
fluke.write("OUT 0.0 V, 0 Hz")
fluke.write("EXTGUARD OFF")
fluke.write("OPER")
fluke.write("RANGELCK ON")
time.sleep(2)
print "MFC configured"

# Now configure all DMMs
dmm.write("BEEP")
dmm.write("END ALWAYS")
dmm.write("DCV 10")
dmm.write("NPLC 10")
dmm.write("NDIG 8")
dmm.write("TRIG AUTO")
dmm.write("MATH OFF")
print "DUT configured, starting ACAL"

dmm2.write("BEEP")
dmm2.write("END ALWAYS")
dmm2.write("DCV 10")
dmm2.write("NPLC 10")
dmm2.write("NDIG 8")
dmm2.write("TRIG AUTO")
dmm2.write("MATH OFF")
print "DUT2 configured, starting ACAL"

dmm3.write("BEEP")
dmm3.write("END ALWAYS")
dmm3.write("DCV 10")
dmm3.write("NPLC 200")
dmm3.write("NDIG 8")
dmm3.write("TRIG AUTO")
dmm3.write("MATH OFF")
print "DUT3 configured, starting ACAL"

dmm4.write("BEEP")
dmm4.write("END ALWAYS")
dmm4.write("DCV 10")
dmm4.write("NPLC 200")
dmm4.write("NDIG 8")
dmm4.write("TRIG AUTO")
dmm4.write("MATH OFF")
print "DUT4 configured, starting ACAL"

dmm.write("ACAL DCV")
dmm2.write("ACAL DCV")
dmm3.write("ACAL DCV")
dmm4.write("ACAL DCV")
time.sleep(160)           # Wait until ACAL DCV is finished

print ("ACAL Finished")

with open(log_name, 'a') as dile:
    dile.write ("date;source;duta;dutb;dutc;dutd;sdev;sdev2;sdev3;sdev4;\r\n")
dile.close()

vcnt = 0 # temporary counter
print "Log into %s started" % log_name

for i in range(-109,110,1):
    median = 0.0
    sdev = 0.0
    array = []
    median2 = 0.0
    sdev2 = 0.0
    array2 = []
    median3 = 0.0
    sdev3 = 0.0
    array3 = []
    median4 = 0.0
    sdev4 = 0.0
    array4 = []
    set_value = (float(i) / 10)    # Calculate set voltage per step
    fluke.write("OUT %.7f" % set_value)
    fluke.write("OPER")
    time.sleep(10)
    for ix in range (0,8): #take more samples
        dmm.write("TARM SGL")
        dmm2.write("TARM SGL")
        dmm3.write("TARM SGL")
        dmm4.write("TARM SGL")
        val = float(dmm.read())
        val2 = float(dmm2.read())
        val3 = float(dmm3.read())
        val4 = float(dmm4.read())
        array.extend([val])
        array2.extend([val2])
        array3.extend([val3])
        array4.extend([val4])
        if abs(set_value) >= 0.05:
            print("\033[33;1mA %.8f, ppm %.4f \033[35;1mB %.8f, ppm %.4f \033[37;1mC %.8f, ppm %.4f \033[32;1mD %.8f, ppm %.4f\033[39;0m X%.8f, ppm %.4f | F %.4f\033[39;0m" % (val, ((val / set_value - 1) * 1e6), val2,((val2 /set_value - 1) * 1e6), val3,((val3 / set_value - 1) * 1e6),val4,((val4 / set_value - 1) * 1e6) ))
            if (abs(val) >= 9.9e36) or (abs(val2) >= 9.9e36) or (abs(val3) >= 9.9e36) or (abs(val4) >= 9.9e36):
            print ("Smoke escaped, check connections")
            fluke.write("OUT 0 V")
            fluke.write("STBY")
            quit()
    sdev = np.std(array[3:])
    median = np.median(array[5:])
    sdev2 = np.std(array2[3:])
    median2 = np.median(array2[5:])
    sdev3 = np.std(array3[3:])
    median3 = np.median(array3[5:])
    sdev4 = np.std(array4[3:])
    median4 = np.median(array4[5:])
    if abs(float(i) / 10) >= 0.0005:
        dev1 = ((median / set_value - 1) * 1e6)
        dev2 = ((median2 /set_value - 1) * 1e6)
        dev3 = ((median3 /set_value - 1) * 1e6)
        dev4 = ((median4 /set_value - 1) * 1e6)
        print("\033[32;1mSRC %.4f \033[33;1mA %.9f, sdev5 = %.3f uV, med5 = %.8f\033[35;1mB %.9f, sdev5 = %.3f uV, med5 = %.8f \033[34;1m Delta %.4f ppm \033[37;1mC %.9f, sdev5 = %.3f uV, med5 = %.8f\033[32;1m D %.9f, sdev5 = %.3f uV, med5 = %.8f\033[39;0m" % (set_value, val, sdev*1e6, median, val2,sdev2*1e6, median2, (dev1 - dev2),  val3,sdev3*1e6, median3, val4,sdev4*1e6, median4 ))
    with open(log_name, 'a') as dile:
        dile.write (time.strftime("%m%d%Y-%H:%M:%S;") + ("%.9f;%.9f;%.9f;%.9f;%.9f;%.6f;%.6g;%.6g;%.6g;\r\n" % (set_value, median, median2, median3, median4, sdev, sdev2, sdev3, sdev4)))
    dile.close()

fluke.write("OUT 0 V")
fluke.write("STBY")
fluke.write("RANGELCK OFF")
dmm.write("RESET")
dmm2.write("RESET")
dmm3.write("RESET")
dmm4.write("RESET")
